
package io.platform.sys.web.controller.rest;


import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.platform.base.common.annotation.SysLog;
import io.platform.base.common.controller.AbstractController;
import io.platform.base.common.utils.Constant;
import io.platform.base.common.utils.ExecuteResult;
import io.platform.base.common.utils.R;
import io.platform.base.common.utils.ShiroUtils;
import io.platform.base.common.validator.Assert;
import io.platform.base.common.validator.ValidatorUtils;
import io.platform.sys.dto.SysUserDto;
import io.platform.sys.web.query.PasswordQuery;
import io.platform.sys.web.remote.SysUserRoleServiceRemote;
import io.platform.sys.web.remote.SysUserServiceRemote;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.apache.shiro.crypto.hash.Sha256Hash;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;


@RestController
@RequestMapping("/sys/user")
public class SysUserRestController extends AbstractController {

    @Autowired
    private SysUserServiceRemote sysUserServiceRemote;
    @Autowired
    private SysUserRoleServiceRemote sysUserRoleServiceRemote;

    /**
     * 所有列表
     */
    @RequestMapping("/list")
    @RequiresPermissions("sys:user:list")
    public R list(Page<SysUserDto> page, SysUserDto sysUserDto) {
        //只有超级管理员，才能查看所有管理员列表
        if (getUserId() != Constant.SUPER_ADMIN) {
            sysUserDto.setCreateUserId(getUserId());
        }
        return R.ok().put("page", sysUserServiceRemote.getPage(page, sysUserDto));
    }

    /**
     * 分页列表
     */
    @RequestMapping("/page")
    @RequiresPermissions("sys:user:page")
    public R page(Page page, SysUserDto sysUserDto) {

        return R.ok().put("page", sysUserServiceRemote.getPage(page, sysUserDto));
    }

    /**
     * 信息
     */
    @RequestMapping("/info")
    @RequiresPermissions("sys:user:info")
    public R info() {

        return R.ok().put("user", ShiroUtils.getUserEntity());
    }

    /**
     * 用户信息
     */
    @GetMapping("/info/{userId}")
    @RequiresPermissions("sys:user:info")
    public R info(@PathVariable("userId") Long userId) {
        SysUserDto user = sysUserServiceRemote.getOne(userId);

        //获取用户所属的角色列表
        List<Long> roleIdList = sysUserRoleServiceRemote.queryRoleIdList(userId);
        user.setRoleIdList(roleIdList);

        return R.ok().put("user", user);
    }

    /**
     * 信息
     */
    @RequestMapping("/detail")
    @RequiresPermissions("sys:user:info")
    public R detail(Long id) {
        SysUserDto sysUserDto = sysUserServiceRemote.getOne(id);
        return R.ok().put("user", sysUserDto);
    }

    /**
     * 保存
     */
    @SysLog("保存")
    @RequestMapping("/save")
    @RequiresPermissions("sys:user:save")
    public R save(@RequestBody SysUserDto sysUserDto) {
        ValidatorUtils.validateEntity(sysUserDto);
        ExecuteResult<String> result = sysUserServiceRemote.save(sysUserDto);
        if (result.isSuccess()) {
            return R.ok(result.getSuccessMessage());
        } else {
            return R.error(result.getErrorMessages().get(0));
        }
    }

    /**
     * 修改
     */
    @SysLog("修改")
    @RequestMapping("/update")
    @RequiresPermissions("sys:user:update")
    public R update(@RequestBody SysUserDto sysUserDto) {
        ValidatorUtils.validateEntity(sysUserDto);
        ExecuteResult<String> result = sysUserServiceRemote.update(sysUserDto);
        if (result.isSuccess()) {
            return R.ok(result.getSuccessMessage());
        } else {
            return R.error(result.getErrorMessages().get(0));
        }
    }

    /**
     * 删除
     */
    @SysLog("删除")
    @RequestMapping("/delete")
    @RequiresPermissions("sys:user:delete")
    public R delete(@RequestBody List<Long> ids) {
        ExecuteResult<String> result = sysUserServiceRemote.deleteBatch(ids);
        if (result.isSuccess()) {
            return R.ok(result.getSuccessMessage());
        } else {
            return R.error(result.getErrorMessages().get(0));
        }
    }

    /**
     * 修改登录用户密码
     */
    @SysLog("修改密码")
    @PostMapping("/password")
    public R password(@RequestBody PasswordQuery form) {
        Assert.isBlank(form.getNewPassword(), "新密码不为能空");

        //sha256加密
        String password = new Sha256Hash(form.getPassword(), getUser().getSalt()).toHex();
        //sha256加密
        String newPassword = new Sha256Hash(form.getNewPassword(), getUser().getSalt()).toHex();

        //更新密码
        boolean flag = sysUserServiceRemote.updatePassword(getUserId(), password, newPassword);
        if (!flag) {
            return R.error("原密码不正确");
        }

        return R.ok();
    }

}
