package io.platform.base.common.utils;

import java.io.UnsupportedEncodingException;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Set;

import com.auth0.jwt.JWT;
import com.auth0.jwt.JWTVerifier;
import com.auth0.jwt.JWTCreator.Builder;
import com.auth0.jwt.algorithms.Algorithm;
import com.auth0.jwt.exceptions.JWTVerificationException;
import com.auth0.jwt.interfaces.DecodedJWT;

/**
 * https://jwt.io/
 * 使用JsonWebToken进行token的创建
 * 和token的校验，
 * com/b2b/common/createToken.java 类已经放弃
 * 
 * @since 2019.06.17
 * @author zhouhui
 * */
public class JsonWebToken {
	private String TOKENKEY = "";
	private String ISSUSER = "";
	private String ENCODETYPE = "HS256";
	
	/**
	 * @param tokenKey 加密的密钥
	 * 
	 * 构造方法
	 * @since 2019.06.17
	 * @author zhouhui
	 * @throws Exception 
	 * */
	public JsonWebToken(String tokenKey) throws Exception{
		if("".equals(tokenKey) || tokenKey == null){
			throw new Exception("未设置密钥");
		}
		this.TOKENKEY = tokenKey;
		this.ISSUSER = null;
	}
	
	/**
	 * @param tokenKey 加密的密钥
	 * @param issuser 签名
	 * 
	 * 构造方法
	 * @since 2019.06.17
	 * @author zhouhui
	 * @throws Exception 
	 * */
	public JsonWebToken(String tokenKey, String issuser) throws Exception{
		if("".equals(tokenKey) || tokenKey == null){
			throw new Exception("未设置密钥");
		}
		if("".equals(issuser) || issuser == null){
			throw new Exception("未设置签名");
		}
		this.TOKENKEY = tokenKey;
		this.ISSUSER = issuser;
	}
	
	/**
	 * @param tokenKey 加密的密钥
	 * @param issuser 签名
	 * @param 
	 * 
	 * 构造方法
	 * @since 2019.06.17
	 * @author zhouhui
	 * @throws Exception 
	 * */
	public JsonWebToken(String tokenKey, String issuser,String encodeType) throws Exception{
		if("".equals(tokenKey) || tokenKey == null){
			throw new Exception("未设置密钥。");
		}
		if("".equals(issuser) || issuser == null){
			throw new Exception("未设置签名。");
		}
		if("".equals(encodeType) || encodeType == null){
			throw new Exception("未设置加密方式。");
		}
		this.TOKENKEY = tokenKey;
		this.ISSUSER = issuser;
		this.ENCODETYPE = encodeType;
	}
	
	/**
	 * 通过传入的参数生成对应的token
	 * @param option token中包含的参数
	 * @return String 生成的token数据
	 * 
	 * @since 2019.06.17
	 * @author zhouhui
	 * */
	public String createToken(HashMap<String, String> option,boolean isUseIss) throws IllegalArgumentException, UnsupportedEncodingException{
		String TOKEN = "";
		Builder build = null;
		if(isUseIss){
			build = JWT.create().withIssuer(ISSUSER);
		}else{
			build = JWT.create();
		}
		
		Set<String> keyList = option.keySet();
		Iterator<String> keyIter = keyList.iterator();
		while(keyIter.hasNext()){
			String key = keyIter.next();
			String val = option.get(key);
			build.withClaim(key, val);
		}
		Algorithm algorithm = null;
		if("HS256".equals(ENCODETYPE)){
			algorithm = Algorithm.HMAC256(TOKENKEY);
		}
		if(algorithm == null){
			throw new UnsupportedEncodingException("\""+ENCODETYPE+"\"不支持此加密方式。");
		}
		TOKEN = build.sign(algorithm);
		return TOKEN;
	}
	
	/**
	 * 判断生成的token是否正确
	 * @param token 需要校验的token信息
	 * @return boolean 判断token是否正确 ，true是正确
	 * 
	 * @since 2019.06.17
	 * @author zhouhui
	 * */
	public boolean isRightOFToken(String token,boolean isCheckIss){
		boolean isRight = false;
		if(token != null && token != ""){
			try {
				Algorithm algorithm = null;
				if("HS256".equals(ENCODETYPE)){
					algorithm = Algorithm.HMAC256(TOKENKEY);
				}
				if(algorithm == null){
					throw new UnsupportedEncodingException("\""+ENCODETYPE+"\"不支持此加密方式。");
				}
				JWTVerifier verifier = null;
				if(isCheckIss){
					verifier = JWT.require(algorithm).withIssuer(ISSUSER).build();
				}else{
					verifier = JWT.require(algorithm).build();
				}
				if(verifier == null){
					return false;
				}
				
				DecodedJWT jwt = verifier.verify(token);
				if(jwt != null){
					isRight = true;
				}
			} catch (IllegalArgumentException e) {
				e.printStackTrace();
			} catch (UnsupportedEncodingException e) {
				e.printStackTrace();
			} catch (JWTVerificationException e) {
				e.printStackTrace();
			}
		}
		return isRight;
	}
	
	/**
	 * 将token转成对应的DecodedJWT
	 * @param token 加密的字符串
	 * @return DecodedJWT 转成的对象
	 * 
	 * @since 2019.06.17
	 * @author zhouhui
	 * */
	public DecodedJWT decodeToken(String token,boolean isCheckIss){
		boolean isRight = isRightOFToken(token,isCheckIss);
		if(isRight){
			return JWT.decode(token);
		}else{
			return null;
		}
	}
	
}
